clear;clc;
%% load data
loaddata;

%% MCMC setup
nsims = 20000;                          % retained MCMC draws
burnin = 2000;                          % burnin period            

loff = 4;   % VAR lag offset (i.e., first VAR lag is y(t-loff), etc.)
p = 4;                               % order of VAR
dates = dates(p+loff:end);

% starting date (1970Q1)
startingdate = 1970;
Data = [Ycore Y4others];

% standardized data
Data = (Data - repmat(mean(Data), size(Data,1), 1))./ repmat(std(Data),size(Data,1),1);

% 1: VAR_SV; 2:VAR_CCM2 3:VAR_Homo 4:VAR_CCM1 5:large_BVAR
model = 1;

switch model
    case 1
        %% VAR-SV
        display(' ')
        display( 'VAR-SV' )
        display(' ')

        VAR_SV_forecasting;

        if exist(sprintf('othermodelsforecasting_results'),'file') == 0
            mkdir othermodelsforecasting_results;
        end
        save(sprintf('./othermodelsforecasting_results/7_VAR_SV'),...
            'yhat1','yhat2','yhat3','yhat4','yhat5','yhat6','yhat7','yhat8','yhat9','yhat10','yhat11','yhat12',...
            'Jyhat1','Jyhat2','Jyhat3','Jyhat4','Jyhat5','Jyhat6','Jyhat7','Jyhat8','Jyhat9','Jyhat10','Jyhat11','Jyhat12'); 
        
    case 2
        %% VAR-CCM2
        display(' ')
        display( 'VAR-CCM2' )
        display(' ')

        VAR_CCM2_forecasting;

        if exist(sprintf('othermodelsforecasting_results'),'file') == 0
            mkdir othermodelsforecasting_results;
        end
        save(sprintf('./othermodelsforecasting_results/7_VAR_CCM2'),...
            'yhat1','yhat2','yhat3','yhat4','yhat5','yhat6','yhat7','yhat8','yhat9','yhat10','yhat11','yhat12',...
            'Jyhat1','Jyhat2','Jyhat3','Jyhat4','Jyhat5','Jyhat6','Jyhat7','Jyhat8','Jyhat9','Jyhat10','Jyhat11','Jyhat12');  
        
    case 3
        %% VAR-Homo
        display(' ')
        display( 'VAR-Homo' )
        display(' ')

        VAR_Homo_forecasting;

        if exist(sprintf('othermodelsforecasting_results'),'file') == 0
            mkdir othermodelsforecasting_results;
        end
        save(sprintf('./othermodelsforecasting_results/large_VAR_Homo'),...
            'yhat1','yhat2','yhat3','yhat4','yhat5','yhat6','yhat7','yhat8','yhat9','yhat10','yhat11','yhat12',...
            'Jyhat1','Jyhat2','Jyhat3','Jyhat4','Jyhat5','Jyhat6','Jyhat7','Jyhat8','Jyhat9','Jyhat10','Jyhat11','Jyhat12'); 
        
    case 4
        %% VAR-CCM1
        display(' ')
        display( 'VAR-CCM1' )
        display(' ')

        VAR_CCM1_forecasting;

        if exist(sprintf('othermodelsforecasting_results'),'file') == 0
            mkdir othermodelsforecasting_results;
        end
        save(sprintf('./othermodelsforecasting_results/7_VAR_CCM1_2'),...
            'yhat1','yhat2','yhat3','yhat4','yhat5','yhat6','yhat7','yhat8','yhat9','yhat10','yhat11','yhat12',...
            'Jyhat1','Jyhat2','Jyhat3','Jyhat4','Jyhat5','Jyhat6','Jyhat7','Jyhat8','Jyhat9','Jyhat10','Jyhat11','Jyhat12'); 
        
    case 5
        %% large-BVAR
        Data = [Ycore Yothers];
        Data = (Data - repmat(mean(Data), size(Data,1), 1))./ repmat(std(Data),size(Data,1),1);
        display(' ')
        display( 'large-BVAR' )
        display(' ')
        
        large_BVAR_forecasting;
            
        if exist(sprintf('othermodelsforecasting_results'),'file') == 0
            mkdir othermodelsforecasting_results;
        end
        save(sprintf('./othermodelsforecasting_results/large_BVAR'),...
            'yhat1','yhat2','yhat3','yhat4','yhat5','yhat6','yhat7','yhat8','yhat9','yhat10','yhat11','yhat12',...
            'Jyhat1','Jyhat2','Jyhat3','Jyhat4','Jyhat5','Jyhat6','Jyhat7','Jyhat8','Jyhat9','Jyhat10','Jyhat11','Jyhat12'); 
        
end




