function s_fcst = VARCL_fcst( Y, X1, X2, b_core, g_core, h_core, sg_core, sh_core, ...
                              b_other, g_other, h_other, sg_other, sh_other, ...
                              wCL, beta0, Vbeta, ncore, nother, ...
                              kgam_core, kgam_other, m_all, nsims, loff )
%% generate point, density and CPRS forecast evals from the composite predictive posterior

ttmax = size( Y, 1 );
if loff > 1
    % assume we're doing direct forecasting only
    ttmax = 1;
end

% preconstruct useful indices
icov = triu( repmat( 2:ncore, ncore - 1, 1 ) );
icov = reshape( icov, [], 1 );
icov( icov == 0 ) = [];
jcov = triu( repmat( ( 1: ncore - 1 )', 1, ncore - 1 ) );
jcov = reshape( jcov, [], 1 );
jcov( jcov == 0 ) = [];

% allocate space for storage
s_fcst = nan( nsims, ttmax, ncore + 1, 5 );

% process all saved draws
for isim = 1:nsims
    % initialize data
    fX1 = X1( 1, : ); fX1_alt = fX1;
    fX2 = X2( 1, :, : ); fX2_alt = fX2;
    
    % set up core parameters
    fb_core = b_core( isim, : )';
    fsg_core = sg_core( isim, : )';
    fsh_core = sh_core( isim, : )';
    fg_core = g_core( isim, end - kgam_core + 1:end )'; fg_core_alt = fg_core;
    fh_core = h_core( isim, end - ncore + 1:end )'; fh_core_alt = fh_core;
    
    % set up non-core parameters
    fb_other = b_other( isim, :, : );
    fsg_other = sg_other( isim, :, : );
    fsh_other = sh_other( isim, :, : );
    fg_other = g_other( isim, end - kgam_other + 1:end, : ); fg_other_alt = fg_other;
    fh_other = h_other( isim, end - nother + 1:end, : ); fh_other_alt = fh_other;

    for tt = 1:ttmax
        % generate fcst of ycore_t
        for count = 1:loff
            fg_core = fg_core + sqrt( fsg_core ) .* randn( kgam_core, 1 );
            fh_core = fh_core + sqrt( fsh_core ) .* randn( ncore, 1 );
        end
        
        B = speye( ncore ) + sparse( icov, jcov, fg_core, ncore, ncore );
        fX1b1 = kron( speye( ncore ), fX1 ) * fb_core;
        
        ifSig = 0;
        ifSig_fmu = 0;
        for mi = 1:m_all
            bigfX2 = kron( speye( ncore ), fX2( 1, :, mi ) );
            Vy = sparse( 1:ncore, 1:ncore, exp( fh_core ) ) + bigfX2 * Vbeta * bigfX2';
            wK = wCL( mi ) * eye( ncore ) / Vy;
            ifSig = ifSig + wK;
            ifSig_fmu =  ifSig_fmu + wK * ( fX1b1 + bigfX2 * beta0 );
        end
        ym = B \ ( ifSig \ ifSig_fmu );
        ifSig = B' * ifSig * B; CifSig = chol( ifSig ); % re-use these later
        yhat = ym + CifSig \ randn( ncore, 1 );
      
        % generate fcst of yother_t (if not at the end of the horizon)
        if tt < ttmax
            for mi = 1:m_all
                for count = 1:loff
                    fg_other = fg_other + sqrt( fsg_other ) .* randn( kgam_other, m_all );
                    fh_other = fh_other + sqrt( fsh_other ) .* randn( nother, m_all );
                end
                fXz = [1; reshape( [reshape( fX1( 2:end ), ncore, [] ); ...
                        reshape( fX2( :, :, mi ), nother, [] )], [], 1 )]';

                % generate a draw of the extra variable (note: this only works
                % for nother = 1; needs to be recoded if nother > 1)
                zhat = fg_other( :, mi )' * yhat + fXz * fb_other ...
                        + exp( ( fh_other( :, mi ) - log( wCL( mi ) ) ) / 2 ) * randn;

                % update lags of non-core variable(s)
                fX2( :, :, mi ) = [zhat' fX2( 1:end - nother )];
            end
        end
        
        % update lags of core variables
        fX1 = [1 yhat' fX1( 2:end - ncore ) ];

        %% repeat for CPRS
        % generate fcst of ycore_t
        for count = 1:loff
            fg_core_alt = fg_core_alt + sqrt( fsg_core ) .* randn( kgam_core, 1 );
            fh_core_alt = fh_core_alt + sqrt( fsh_core ) .* randn( ncore, 1 );
        end
        
        B = speye( ncore ) + sparse( icov, jcov, fg_core_alt, ncore, ncore );
        fX1b1 = kron( speye( ncore ), fX1_alt ) * fb_core;
        
        ifSig_alt = 0;
        ifSig_fmu = 0;
        for mi = 1:m_all
            bigfX2 = kron( speye( ncore ), fX2_alt( 1, :, mi ) );
            Vy = sparse( 1:ncore, 1:ncore, exp( fh_core_alt ) ) + bigfX2 * Vbeta * bigfX2';
            wK = wCL( mi ) * eye( ncore ) / Vy;
            ifSig_alt = ifSig_alt + wK;
            ifSig_fmu =  ifSig_fmu + wK * ( fX1b1 + bigfX2 * beta0 );
        end
        yhat_alt = B \ ( ifSig_alt \ ifSig_fmu + chol( ifSig_alt ) \ randn( ncore, 1 ) );
      
        % generate fcst of yother_t (if not at the end of the horizon)
        if tt < ttmax
            for mi = 1:m_all
                for count = 1:loff
                    fg_other_alt = fg_other_alt + sqrt( fsg_other ) .* randn( kgam_other, m_all );
                    fh_other_alt = fh_other_alt + sqrt( fsh_other ) .* randn( nother, m_all );
                end
                fXz = [1; reshape( [reshape( fX1_alt( 2:end ), ncore, [] ); ...
                        reshape( fX2_alt( :, :, mi ), nother, [] )], [], 1 )]';

                % generate a draw of the extra variable (note: this only works
                % for nother = 1; needs to be recoded if nother > 1)
                zhat = fg_other_alt( :, mi )' * yhat_alt + fXz * fb_other ...
                        + exp( ( fh_other_alt( :, mi ) - log( wCL( mi ) ) ) / 2 ) * randn;

                % update lags of non-core variable(s)
                fX2_alt( :, :, mi ) = [zhat' fX2_alt( 1:end - nother )];
            end
        end
        
        % update lags of core variables
        fX1_alt = [1 yhat_alt' fX1_alt( 2:end - ncore ) ];

        %% compute the marginal variances for ycore
        lndet_ifSig = 2 * sum( log( diag( CifSig ) ) );
        if ncore == 3
            dfSig = [ifSig( 2, 2 ) * ifSig( 3, 3 ) - ifSig( 2, 3 ) * ifSig( 3, 2 ); ...
                     ifSig( 1, 1 ) * ifSig( 3, 3 ) - ifSig( 1, 3 ) * ifSig( 3, 1 ); ...
                     ifSig( 1, 1 ) * ifSig( 2, 2 ) - ifSig( 1, 2 ) * ifSig( 2, 1 )] ...
                     * exp( -lndet_ifSig );
        else
            dfSig = diag( inv( ifSig ) );
        end
        
        %% generate forecast statistics
        s_fcst( isim, tt, :, : ) = [[yhat; nan] [yhat_alt; nan] ...
            [( yhat - Y( tt, : )' ) .^ 2; nan] ...
            [abs( yhat - Y( tt, : )' ) - .5 * abs( yhat - yhat_alt ); nan] ...
            exp( [-0.5 * ( log( dfSig ) + ( Y( tt, : )' - ym ) .^ 2 ./ dfSig ); ...
            0.5 * ( lndet_ifSig - sum( ( CifSig * ( Y( tt, : )' - ym ) ) .^ 2 ) )] )];
    end
end