function [s_beta, s_gam, s_h, s_Siggam, s_Sigh, s_gam0, s_h0] ...
            = VARSV_mcmc( y, X, W, T, n, stn, kbeta, kgam, ...
                beta0, Vbeta, agam, Vgam, ah, Vh, nugam0, Sgam0, nuh0, Sh0, ...
                nsims, burnin, svsims, simstep, startvals )

%% initialize
if nargin < 23
    beta = X \ y;
    Siggam = .01 * ones( kgam, 1 );
    Sigh = .01 * ones( n, 1 );
    h0 = log( var( reshape( y, n, T ), 0 , 2 ) );
    h = repmat( h0', T, 1 );
    gam0 = zeros( kgam, 1 );
else
    beta = startvals.beta;
    Siggam = startvals.Siggam;
    Sigh = startvals.Sigh;
    h0 = startvals.h0;
    h = startvals.h;
    gam0 = startvals.gam0;
end
    
%% allocate space for draws
s_Siggam = zeros( svsims, kgam ); 
s_Sigh = zeros( svsims, n );
s_beta = zeros( svsims, kbeta ); 
s_gam = zeros( svsims, T * kgam ); 
s_gam0 = zeros( svsims, kgam );
s_h = zeros( svsims, T * n );
s_h0 = zeros( svsims, n );
    
%% outside the loop
Hgam = speye( T * kgam ) - sparse( kgam + 1:T * kgam, 1:( T - 1 ) * kgam, ...
                            ones( ( T - 1 ) * kgam, 1 ), T * kgam, T * kgam );
alpgam =  Hgam \ [gam0; sparse( ( T - 1 ) * kgam, 1 )];   
    
for isim = 1:nsims + burnin    
	%% sample gam    
	invSig = sparse( 1:T * n, 1:T * n, reshape( exp( -h' ), T * n, 1 ) ); 
	invS = sparse( 1:T * kgam, 1:T * kgam, repmat( 1 ./ Siggam', 1, T ) );
	WinvSig = W' * invSig;
	HinvSH = Hgam' * invS * Hgam;
	Kgam = HinvSH + WinvSig * W;
	gamhat = Kgam \ ( HinvSH * alpgam + WinvSig * ( y - X * beta ) );
    gam = gamhat + chol( Kgam ) \ randn( T * kgam, 1 );

    %% sample gam0 (note: we should probably sample gam and gam0 jointly)
	Kgam0 = sparse( 1:kgam, 1:kgam, 1 ./ Siggam + 1 ./ Vgam );
	gam0hat = Kgam0 \ ( agam ./ Vgam + gam( 1:kgam ) ./ Siggam );
	gam0 = gam0hat + chol( Kgam0 ) \ randn( kgam, 1 );
    
	%% sample Siggam
	e = reshape( gam - [gam0; gam( 1:( T - 1 ) * kgam )], kgam, T );
	Siggam = 1 ./ gamrnd( nugam0 + T / 2, 1 ./ ( Sgam0 + sum( e .^ 2, 2 ) / 2 ) );

	%% sample beta
	XinvSig = X' * invSig;
	Kbeta = sparse( 1:kbeta, 1:kbeta, 1 ./ Vbeta ) + XinvSig * X;
	betahat = Kbeta \ ( beta0 ./ Vbeta + XinvSig * ( y - W * gam ) );
	beta = betahat + chol( Kbeta ) \ randn( kbeta, 1 );

    if ~isempty( stn )
        % force stationarity (possibly for a subset of variables)
        Beta = reshape( beta( stn.idx ), [], stn.n )';
        Comp = [Beta( :, 2:end ); eye( stn.n * ( stn.p - 1 ) ) ...
            zeros( stn.n * ( stn.p - 1 ) , stn.n )];

        count = 0;
        while isim > max( 0.01 * burnin, 100 ) && max( abs( eig( Comp ) ) > 1 )
            beta = betahat + chol( Kbeta ) \ randn( kbeta, 1 );
            Beta = reshape( beta( stn.idx ), [], stn.n )';
            Comp = [Beta( :, 2:end ); eye( stn.n * ( stn.p - 1 ) ) ...
                zeros( stn.n * ( stn.p - 1 ) , stn.n )];
            count = count + 1;
        end
    end
    
	%% sample h  
	u = y - X * beta - W * gam;
	shortu = reshape( u, n, T )';
	for i = 1:n
        Ystar = log( shortu( :, i ) .^ 2 + .0001 );
        h( :, i ) = SVRW( Ystar, h( :, i ), Sigh( i ), h0( i ) );
	end    

	%% sample h0 (note: we should probably sample h and h0 jointly)
	Kh0 = sparse( 1:n, 1:n, 1 ./ Sigh + 1 ./ Vh );
	h0hat = Kh0 \ ( ah ./ Vh + h( 1, : )' ./ Sigh );
	h0 = h0hat + chol( Kh0 ) \ randn( n, 1 );

	%% sample Sigh
	e = h - [h0'; h( 1:T - 1, : )];
	Sigh = 1 ./ gamrnd( nuh0 + T / 2, 1 ./ ( Sh0 + sum( e .^ 2 )' / 2 ) );

    if isim > burnin && mod( isim - burnin, simstep ) == 0
        isave = ( isim - burnin ) / simstep;            

        % store the draws
        s_beta( isave, : ) = beta';
        s_gam( isave, : ) = gam';
        s_h( isave, : ) = reshape( h', 1, n * T );
        s_Siggam( isave, : ) = Siggam';
        s_Sigh( isave, : ) = Sigh';
        s_gam0( isave, : ) = gam0';
        s_h0( isave, : ) = h0';
    end
end