function [wCL, GA_pden] = comp_CL_weights( simresults, fcst_t, GA_pden )
%% compute the composite likelihood weights

m_all = length( fcst_t ) - 2;
lnML = reshape( [simresults.mlw], 2, m_all );
lnDIC = reshape( [simresults.dic], 4, m_all );
fcstdraws = cat( 5, fcst_t{1:m_all} );
llikedraws = squeeze( fcstdraws( :, 1, end, end, : ) );

% ML-based weights
maxlnML = max( lnML( 1, : ) );
wML = exp( lnML( 1, : ) - maxlnML ) / sum( exp( lnML( 1, : ) - maxlnML ) );

% DIC weights
maxlnDIC = max( lnDIC( 1, : ) );
wDIC = exp( lnDIC( 1, : ) - maxlnDIC ) / sum( exp( lnDIC( 1, : ) - maxlnDIC ) );

% BIC weights
maxlnBIC = max( lnDIC( 4, : ) ); % maxllike is the fourth row in lnDIC
wBIC = exp( lnDIC( 4, : ) - maxlnBIC ) / sum( exp( lnDIC( 4, : ) - maxlnBIC ) );

% Geweke-Amisano weights
wGAt = ones( m_all, 1 ) / m_all;
if ~isempty( GA_pden )
    % compute optimal weights
    wGAt = fmincon( @poolscore, wGAt, [], [], ones( 1, m_all ), 1, zeros( m_all, 1 ), [], [], ...
       optimset( 'HessFcn', @poolscoreH, 'Algorithm', 'interior-point', ...
       'GradObj', 'on', 'Hessian', 'user-supplied', 'Display', 'Off' ) );
end
maxllikedraws = max( llikedraws, [], 1 );
GA_pden = [GA_pden; mean( exp( bsxfun( @minus, llikedraws, maxllikedraws ) ) )];

% Equal weights
wEQ = ones( m_all, 1 ) / m_all;

wCL = {wML'; wDIC'; wBIC'; wGAt; wEQ};

%% nested function for numerical optimisation
function [f, g, H] = poolscore( w )
    % the objective function

    pw = GA_pden * w;
    p_pw = bsxfun( @rdivide, GA_pden, pw );

    f = -sum( log( pw ) );

    if nargout == 2
        g = -p_pw' * ones( size( GA_pden, 1 ), 1 );
    end

    if nargout == 3
        H = p_pw' * p_pw;
    end
end

function H = poolscoreH( w, ~ )
    % the hessian for interior-point algorithms

    pw = GA_pden * w;
    p_pw = bsxfun( @rdivide, GA_pden, pw );
    H = p_pw' * p_pw;
end

end